package org.jeecg.modules.adweb.seo.controller;

import io.swagger.v3.oas.annotations.tags.Tag;

import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.adweb.seo.dto.ComprehensiveStatistics;
import org.jeecg.modules.adweb.subscription.entity.UserPlanSubscription;
import org.jeecg.modules.adweb.seo.service.ISeoKeywordsRankService;
import org.jeecg.modules.adweb.site.entity.AdwebSite;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.List;
import java.util.Map;

/**
 * @Description: adweb_seo_keywords_rank
 * @Author: jeecg-boot
 * @Date: 2024-10-09
 * @Version: V1.0
 */
@Tag(name = "adweb_seo_keywords_rank")
@RestController
@RequestMapping("/seo/seoKeywordsRank")
@Slf4j
public class SeoKeywordsRankController {
    @Autowired private ISeoKeywordsRankService seoKeywordsRankService;

    /** 获取当前用户管理的 父 站点信息 @CopyFrom: getAllSites */
    @GetMapping("/getAllSites")
    public Result<List<AdwebSite>> getAllSites() {
        final List<AdwebSite> allSites = seoKeywordsRankService.getAllSites();
        return Result.OK(allSites);
    }

    /** 根据 站点code 获取 站点的 订阅Id */
    @GetMapping(value = "/getSubscriptionIdBySiteCode")
    public Result<List<UserPlanSubscription>> getSubscriptionIdBySiteCode(
            @RequestParam String siteCode) {
        final List<UserPlanSubscription> subscriptionIdBySiteCode =
                seoKeywordsRankService.getSubscriptionIdBySiteCode(siteCode);
        return Result.OK(subscriptionIdBySiteCode);
    }

    /** 获得综合统计信息 */
    @GetMapping(value = "/comprehensiveInfo")
    public Result<?> getComprehensiveInfo(@RequestParam String siteCode) throws ParseException {
        ComprehensiveStatistics statistics = seoKeywordsRankService.getComprehensiveInfo(siteCode);
        return Result.OK(statistics);
    }

    /** 获取关键字排名信息 */
    @GetMapping(value = "/getRankInfo")
    public Result<?> getRankInfo(@RequestParam("siteCode") String siteCode) throws ParseException {
        Map map = seoKeywordsRankService.getRankInfo(siteCode);
        return Result.OK(map);
    }
}
